"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const Configuration_1 = require("./Configuration");
const util_1 = require("./util");
const tsioc_1 = require("tsioc");
const path = require("path");
const util_2 = require("util");
const Application_1 = require("./Application");
const router_1 = require("./router");
const registerDefaults_1 = require("./registerDefaults");
const decorators_1 = require("./decorators");
const child_process_1 = require("child_process");
/**
 * Bootstrap
 *
 * @export
 * @class Bootstrap
 */
class Bootstrap {
    /**
     * Creates an instance of WebApplication.
     * @param {string} rootdir
     * @param {Type<any>} [appType]
     * @memberof WebApplication
     */
    constructor(rootdir, appType) {
        this.rootdir = rootdir;
        this.appType = appType;
        this.middlewares = [];
        this.appType = this.appType || Application_1.Application;
    }
    /**
     * create new application.
     *
     * @static
     * @param {string} rootdir
     * @param {Type<Application>} [appType]
     * @returns
     * @memberof WebApplication
     */
    static create(rootdir, appType) {
        return new Bootstrap(rootdir, appType);
    }
    /**
     * user custom IContainer.
     *
     * @param {(IContainer | Promise<IContainer>)} [container]
     * @returns
     *
     * @memberOf WebHostBuilder
     */
    useContainer(container) {
        if (container) {
            if (!this.container) {
                this.container = util_1.Defer.create();
            }
            this.container.resolve(container);
        }
        return this;
    }
    getContainer() {
        if (!this.container) {
            this.useContainer(this.createContainer());
        }
        return this.container.promise;
    }
    createContainer(option) {
        return this.getContainerBuilder().build(option);
    }
    useContainerBuilder(builder) {
        this.builder = builder;
        return this;
    }
    getContainerBuilder() {
        if (!this.builder) {
            this.builder = new tsioc_1.ContainerBuilder();
        }
        return this.builder;
    }
    /**
     * use custom configuration.
     *
     * @param {(string | IConfiguration)} [config]
     * @returns {Bootstrap}
     *
     * @memberOf WebHostBuilder
     */
    useConfiguration(config) {
        if (!this.configDefer) {
            this.configDefer = util_1.Defer.create();
            this.configDefer.resolve(new Configuration_1.Configuration());
        }
        let excfg;
        if (util_2.isString(config)) {
            if (fs_1.existsSync(config)) {
                excfg = require(config);
            }
            else if (child_process_1.execFileSync(path.join(this.rootdir, config))) {
                excfg = require(path.join(this.rootdir, config));
            }
            else {
                console.log(`config file: ${config} not exists.`);
            }
        }
        else if (config) {
            excfg = config;
        }
        else {
            let cfgpath = path.join(this.rootdir, './config');
            let config;
            ['.js', '.json'].forEach(ext => {
                if (config) {
                    return false;
                }
                if (fs_1.existsSync(cfgpath + ext)) {
                    config = require(cfgpath + ext);
                    return false;
                }
                return true;
            });
            if (!config) {
                console.log('your app has not config file.');
            }
            excfg = config;
        }
        if (excfg) {
            this.configDefer.promise = this.configDefer.promise
                .then(cfg => {
                cfg = Object.assign(cfg || {}, excfg || {});
                return cfg;
            });
        }
        return this;
    }
    getConfiguration() {
        if (!this.configDefer) {
            this.useConfiguration();
        }
        return this.configDefer.promise;
    }
    /**
     * use middleware `fn` or  `MiddlewareFactory`.
     * @param {MvcMiddleware} middleware
     * @returns {Bootstrap}
     * @memberOf WebHostBuilder
     */
    use(middleware) {
        this.middlewares.push(middleware);
        return this;
    }
    /**
     * run service.
     * @returns {Application}
     * @memberOf WebHostBuilder
     */
    async run() {
        let app = await this.build();
        let config = app.container.get(Configuration_1.Configuration);
        app.listen(config.port || process.env.PORT);
        console.log('service listen on port: ', config.port || app.env['port']);
        return app;
    }
    /**
     * build application.
     * @returns {Bootstrap}
     * @memberOf WebHostBuilder
     */
    async build() {
        let cfg = await this.getConfiguration();
        let container = await this.getContainer();
        await this.initIContainer(cfg, container);
        let app = container.get(this.appType);
        this.setupMiddwares(app, container, cfg.beforeMiddlewares, cfg.excludeMiddlewares);
        this.setupMiddwares(app, container, cfg.useMiddlewares, cfg.excludeMiddlewares.concat(cfg.afterMiddlewares));
        this.setupRoutes(cfg, container);
        this.setupMiddwares(app, container, cfg.afterMiddlewares, cfg.excludeMiddlewares);
        return app;
    }
    async initIContainer(config, container) {
        config.rootdir = config.rootdir ? tsioc_1.toAbsolutePath(this.rootdir, config.rootdir) : this.rootdir;
        container.registerSingleton(Configuration_1.Configuration, config);
        container.registerSingleton(util_1.symbols.IConfiguration, config);
        this.registerExtendDecorators(container);
        // register self.
        container.register(this.appType);
        // custom use.
        if (this.middlewares) {
            await this.builder.loadModule(container, {
                modules: this.middlewares.filter(m => tsioc_1.isClass(m))
            });
        }
        // custom config.
        if (config.middlewares) {
            let modules = await this.builder.loadModule(container, {
                basePath: config.rootdir,
                files: config.middlewares
            });
            if (!config.useMiddlewares || config.useMiddlewares.length < 1) {
                config.useMiddlewares = this.middlewares.concat(modules);
            }
            else {
                config.useMiddlewares = this.middlewares.concat(config.useMiddlewares);
            }
        }
        // register default
        this.registerDefaults(container);
        if (config.controllers) {
            let controllers = await this.builder.loadModule(container, {
                basePath: config.rootdir,
                files: config.controllers
            });
            if (!config.useControllers || config.useControllers.length < 1) {
                config.useControllers = controllers;
            }
        }
        if (config.aop) {
            let aops = await this.builder.loadModule(container, {
                basePath: config.rootdir,
                files: config.aop
            });
            config.usedAops = aops;
        }
        return container;
    }
    registerExtendDecorators(container) {
        decorators_1.registerDecorators(container);
    }
    registerDefaults(container) {
        registerDefaults_1.registerDefaults(container);
    }
    setupRoutes(config, container) {
        let router = container.get(config.routerMiddlewate || router_1.Router);
        router.register(...config.useControllers);
        router.setup();
    }
    setupMiddwares(app, container, middlewares, excludes) {
        middlewares.forEach(m => {
            if (!m) {
                return;
            }
            if (excludes && excludes.length > 0 && excludes.indexOf(m) > 0) {
                return;
            }
            if (tsioc_1.isClass(m) || util_2.isString(m) || util_2.isSymbol(m)) {
                let middleware = container.get(m);
                if (middleware.setup) {
                    middleware.setup();
                }
            }
            else if (tsioc_1.isFunction(m)) {
                app.use(m);
            }
        });
        return app;
    }
}
exports.Bootstrap = Bootstrap;

//# sourceMappingURL=sourcemaps/Bootstrap.js.map
