"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const BaseRoute_1 = require("./BaseRoute");
const tsioc_1 = require("tsioc");
const util_1 = require("../util");
const decorators_1 = require("../decorators");
const decorators_2 = require("../decorators");
const index_1 = require("../errors/index");
const util_2 = require("util");
const restults_1 = require("../restults");
const RequestMethod_1 = require("../RequestMethod");
const Configuration_1 = require("../Configuration");
const ModelParser_1 = require("./ModelParser");
class ControllerRoute extends BaseRoute_1.BaseRoute {
    constructor(route, controller) {
        super(route);
        this.controller = controller;
    }
    async options(container, ctx, next) {
        try {
            await this.invokeOption(ctx, container, next);
        }
        catch (err) {
            if (err instanceof index_1.HttpError) {
                ctx.status = err.code;
                ctx.message = err.message;
            }
            else {
                ctx.status = 500;
                console.error(err);
            }
        }
    }
    async navigating(container, ctx, next) {
        try {
            if (ctx.method !== 'OPTIONS') {
                await this.invoke(ctx, container, (meta, params, ctrl) => this.createProvider(container, meta, params, ctrl, ctx));
            }
            else {
                throw new index_1.ForbiddenError();
            }
            return next();
        }
        catch (err) {
            if (err instanceof index_1.HttpError) {
                ctx.status = err.code;
                ctx.message = err.message;
            }
            else {
                ctx.status = 500;
                console.error(err);
            }
        }
    }
    async invokeOption(ctx, container, next) {
        let requestOrigin = ctx.get('Origin');
        ctx.vary('Origin');
        if (!requestOrigin) {
            return next();
        }
        let origin = requestOrigin;
        let headersSet = {};
        let set = (key, value) => {
            ctx.set(key, value);
            headersSet[key] = value;
        };
        let config = container.get(Configuration_1.Configuration);
        let options = config.corsOptions || {};
        if (ctx.method !== 'OPTIONS') {
            set('Access-Control-Allow-Origin', origin);
            if (options.credentials === true) {
                set('Access-Control-Allow-Credentials', 'true');
            }
            if (options.exposeHeaders) {
                set('Access-Control-Expose-Headers', options.exposeHeaders);
            }
            if (!options.keepHeadersOnError) {
                return next();
            }
            return next().catch(err => {
                err.headers = Object.assign({}, err.headers, headersSet);
                throw err;
            });
        }
        else {
            let coremeta = this.getCorsMeta(ctx, container, ctx.get('Access-Control-Request-Method'));
            // console.log('coremeta', coremeta);
            if (!coremeta) {
                return next();
            }
            options = Object.assign({}, options, coremeta);
            ctx.set('Access-Control-Allow-Origin', origin);
            if (options.credentials === true) {
                ctx.set('Access-Control-Allow-Credentials', 'true');
            }
            let maxAge = String(options.maxAge);
            if (maxAge) {
                ctx.set('Access-Control-Max-Age', maxAge);
            }
            let allowsM = util_2.isArray(options.allowMethods) ?
                options.allowMethods.map(m => RequestMethod_1.methodToString(m)).filter(m => !!m).join(',')
                : options.allowMethods;
            allowsM = allowsM || 'GET,HEAD,PUT,POST,DELETE,PATCH';
            if (allowsM) {
                ctx.set('Access-Control-Allow-Methods', allowsM);
            }
            let allowH = util_2.isArray(options.allowHeaders) ? options.allowHeaders.filter(m => !!m).join(',') : options.allowHeaders;
            if (!allowH) {
                allowH = ctx.get('Access-Control-Request-Headers');
            }
            if (allowH) {
                ctx.set('Access-Control-Allow-Headers', allowH);
            }
            ctx.status = 204;
        }
    }
    getCorsMeta(ctx, container, reqMethod) {
        if (!reqMethod) {
            return null;
        }
        let methodCors = tsioc_1.getMethodMetadata(decorators_1.Cors, this.controller);
        let method = RequestMethod_1.parseRequestMethod(reqMethod);
        let meta = this.getRouteMetaData(ctx, container, method);
        if (meta && meta.propertyKey) {
            let corsmetas = tsioc_1.getMethodMetadata(decorators_1.Cors, this.controller)[meta.propertyKey] || [];
            if (corsmetas.length < 1) {
                corsmetas = tsioc_1.getTypeMetadata(decorators_1.Cors, this.controller);
            }
            if (corsmetas.length) {
                return corsmetas.find(cor => {
                    console.log('find CorsMetadata:', cor);
                    if (!cor.allowMethods) {
                        return true;
                    }
                    if (cor.allowMethods) {
                        if (util_2.isString(cor.allowMethods)) {
                            return cor.allowMethods.toUpperCase().indexOf(reqMethod) >= 0;
                        }
                        else if (util_2.isArray(cor.allowMethods)) {
                            return cor.allowMethods.some(c => RequestMethod_1.methodToString(c) === reqMethod);
                        }
                    }
                    return false;
                });
            }
        }
        return null;
    }
    async invoke(ctx, container, provider) {
        let meta = this.getRouteMetaData(ctx, container, RequestMethod_1.parseRequestMethod(ctx.method));
        if (meta && meta.propertyKey) {
            let ctrl = container.get(this.controller);
            if (container.has(util_1.symbols.IAuthorization)) {
                let hasAuth = Reflect.hasMetadata(decorators_2.Authorization.toString(), ctrl, meta.propertyKey);
                if (hasAuth) {
                    let auth = container.get(util_1.symbols.IAuthorization);
                    if (!auth.isAuth()) {
                        throw new index_1.UnauthorizedError();
                    }
                }
            }
            let lifeScope = container.getLifeScope();
            let params = lifeScope.getMethodParameters(this.controller, ctrl, meta.propertyKey);
            let response = await container.invoke(this.controller, meta.propertyKey, ctrl, ...provider(meta, params, ctrl));
            let contentType = meta.contentType;
            if (util_2.isString(response)) {
                contentType = contentType || 'text/html';
                ctx.response.body = response;
            }
            else if (util_2.isArray(response) && response instanceof Uint8Array) {
                ctx.response.body = Buffer.from(response);
            }
            else if (util_2.isObject(response)) {
                if (response instanceof restults_1.ResultValue) {
                    await response.sendValue(ctx, container);
                }
                else {
                    contentType = contentType || 'application/json';
                    ctx.type = contentType;
                    ctx.response.body = response;
                }
            }
        }
        else {
            throw new index_1.NotFoundError();
        }
    }
    createProvider(container, meta, params, ctrl, ctx) {
        let parser = container.get(ModelParser_1.ModelParser);
        if (params && params.length) {
            let paramVal = null;
            if (this.isRestUri(meta.route)) {
                let route = meta.route.substring(0, meta.route.indexOf('/:')) + '/';
                let baseURL = this.cutEmptyPath(this.url, true);
                let routeUrl = this.cutEmptyPath(ctx.url.replace(baseURL, ''));
                let querystring = routeUrl.replace(route, '');
                paramVal = querystring.indexOf('/') > 0 ? querystring.substring(0, querystring.indexOf('/')) : querystring;
            }
            let body = ctx.request['body'] || {};
            let providers = params.map((p, idx) => {
                try {
                    if (tsioc_1.isClass(p) && Reflect.hasOwnMetadata(decorators_1.Model.toString(), p)) {
                        let val = parser.parseModel(p, body);
                        return {
                            value: val,
                            index: idx
                        };
                    }
                    else if (parser.isBaseType(p)) {
                        let val = parser.parseBaseType(p, paramVal);
                        return {
                            value: val,
                            index: idx
                        };
                    }
                    else {
                        return null;
                    }
                }
                catch (err) {
                    throw new index_1.BadRequestError(err.toString());
                }
            });
            return providers.filter(p => p !== null);
        }
        return [];
    }
    isRestUri(uri) {
        return /\/:/.test(uri || '');
    }
    getRouteMetaData(ctx, container, requestMethod) {
        let decoratorName = decorators_1.Route.toString();
        let baseURL = this.cutEmptyPath(this.url, true);
        let routPath = this.cutEmptyPath(ctx.url.replace(baseURL, ''));
        let methodMaps = tsioc_1.getMethodMetadata(decoratorName, this.controller);
        let meta;
        let allMethods = [];
        for (let name in methodMaps) {
            allMethods = allMethods.concat(methodMaps[name]);
        }
        allMethods = allMethods.filter(m => m && m.method === requestMethod);
        allMethods = allMethods.sort((ra, rb) => (rb.route || '').length - (ra.route || '').length);
        meta = allMethods.find(route => (route.route || '') === routPath);
        if (!meta) {
            meta = allMethods.find(route => {
                let uri = route.route || '';
                // if (uri === routPath) {
                //     return true;
                // }
                if (this.isRestUri(uri)) {
                    let idex = uri.indexOf('/:');
                    let url = uri.substring(0, idex);
                    if (url !== routPath && routPath.indexOf(url) === 0) {
                        return true;
                    }
                }
                return false;
            });
        }
        return meta;
    }
}
exports.ControllerRoute = ControllerRoute;

//# sourceMappingURL=../sourcemaps/router/ControllerRoute.js.map
